/*
 * Copyright 2006-2008 The FLWOR Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "context/root_static_context.h"
#include "context/namespace_context.h"
#include "context/uri_resolver_wrapper.h"
#include "context/standard_uri_resolvers.h"

#include "types/root_typemanager.h"

#include "system/globalenv.h"

#include "compiler/parser/query_loc.h"

#include "common/common.h"

namespace zorba 
{

static const char* default_ns_initializers [] = 
{
  "fn", XQUERY_FN_NS,
  "xml", XML_NS,
  "xs", XML_SCHEMA_NS,
  "xsi", XSI_NS,
  "local", XQUERY_LOCAL_FN_NS,
  "err", XQUERY_ERR_NS,
  NULL, NULL
};


root_static_context::root_static_context() : static_context()
{
  theTypemgr = new RootTypeManager();
}


void root_static_context::init() 
{
  QueryLoc loc;

  theImplementationBaseUri = new xqpStringStore("http://www.zorba-xquery.com/");
  compute_base_uri();

  // default xquery version is 1.1
  set_xquery_version(StaticContextConsts::xquery_version_1_1);
  set_xpath_compatibility(StaticContextConsts::xpath2_0);

  const char** p = default_ns_initializers;
  for (; *p != NULL; p += 2)
  {
    xqpStringStore_t pfx = new xqpStringStore(p[0]);
    xqpStringStore_t ns = new xqpStringStore(p[1]);
    bind_ns(pfx, ns, loc);
  }

  xqpStringStore_t elem_ns = new xqpStringStore("");
  set_default_elem_type_ns(elem_ns, loc);		

  xqpStringStore_t fn_ns = new xqpStringStore(XQUERY_FN_NS);
  set_default_function_ns(fn_ns, loc);

  set_context_item_type(GENV_TYPESYSTEM.ITEM_TYPE_ONE);

  add_collation(ZORBA_DEF_COLLATION_NS, QueryLoc::null);
  add_collation(W3C_CODEPT_COLLATION_NS, QueryLoc::null);
  set_default_collation(W3C_CODEPT_COLLATION_NS, QueryLoc::null);

  set_construction_mode(StaticContextConsts::cons_preserve);
  set_inherit_mode(StaticContextConsts::inherit_ns);
  set_preserve_mode(StaticContextConsts::preserve_ns);

  set_ordering_mode(StaticContextConsts::ordered);

  set_empty_order_mode(StaticContextConsts::empty_greatest);

  set_boundary_space_mode(StaticContextConsts::strip_space);

  set_default_w3c_collection_type(GENV_TYPESYSTEM.ITEM_TYPE_STAR);

  // TODO move into globalenv
  set_document_uri_resolver(new StandardDocumentURIResolver());
  set_collection_uri_resolver(new StandardCollectionURIResolver());
  set_validation_mode(StaticContextConsts::lax_validation);

  std::vector<std::string> lRootModulePaths;
  const char ** lPathsIter = get_builtin_module_paths();
  for (; *lPathsIter != 0; ++lPathsIter) 
  {
    lRootModulePaths.push_back(*lPathsIter);
  }
  set_module_paths(lRootModulePaths);

} 


root_static_context::~root_static_context()
{
}

}
/* vim:set ts=2 sw=2: */
