/*
 * Copyright 2006-2008 The FLWOR Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "store/api/iterator.h"

#include "compiler/xqddf/collection_decl.h"
#include "compiler/xqddf/value_index.h"
#include "compiler/xqddf/value_ic.h"
#include "compiler/expression/var_expr.h"

#include "context/static_context.h"

#include "types/typeimpl.h"

#include "functions/function.h"

#define ZORBA_HASHMAP_WITH_SERIALIZATION
#include "util/hashmap.h"
#undef ZORBA_HASHMAP_WITH_SERIALIZATION

#define ZORBA_UTILS_HASHMAP_WITH_SERIALIZATION
#include "zorbautils/hashmap.h"
#include "zorbautils/hashmap_itemp.h"
#include "zorbautils/hashmap_strh.h"
#undef ZORBA_UTILS_HASHMAP_WITH_SERIALIZATION

#include "zorbaserialization/serialization_engine.h"

#include "zorbatypes/rchandle.h"
#include "zorbatypes/xqpstring.h"

#include "common/shared_types.h"


namespace zorba{

SERIALIZABLE_CLASS_VERSIONS(RCObject)
END_SERIALIZABLE_CLASS_VERSIONS(RCObject)

SERIALIZABLE_CLASS_VERSIONS(SimpleRCObject)
END_SERIALIZABLE_CLASS_VERSIONS(SimpleRCObject)

//SERIALIZABLE_TEMPLATE_VERSIONS(rchandle)
////CLASS_VERSION(2, ZORBA_VERSION_0_9_5, BACKWARD_COMPATIBLE, "add a new version just for test")
//END_SERIALIZABLE_TEMPLATE_VERSIONS(rchandle)
//SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(rchandle, rchandle  <xqpStringStore*>)

SERIALIZABLE_TEMPLATE_VERSIONS(serializable_hashmap)
END_SERIALIZABLE_TEMPLATE_VERSIONS(serializable_hashmap)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_hashmap, serializable_hashmap<xqp_string>, 2)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_hashmap, serializable_hashmap<xqtref_t>, 3)

SERIALIZABLE_TEMPLATE_VERSIONS(serializable_hashmap_entry)
END_SERIALIZABLE_TEMPLATE_VERSIONS(serializable_hashmap_entry)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_hashmap_entry, serializable_hashmap<xqp_string>::entry, 2)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_hashmap_entry, serializable_hashmap<xqtref_t>::entry, 3)


//
// HashMap serialization
//

SERIALIZABLE_TEMPLATE_VERSIONS(serializable_HashEntry)
END_SERIALIZABLE_TEMPLATE_VERSIONS(serializable_HashEntry)

SERIALIZABLE_TEMPLATE_VERSIONS(serializable_HashMap)
END_SERIALIZABLE_TEMPLATE_VERSIONS(serializable_HashMap)

// ItemPointerHashMapCmp
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, StaticallyKnownCollection_t>, 1)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, IndexDecl_t>, 2)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, ValueIC_t>, 3)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, function_t>, 4)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, std::vector<function_t>* >, 5)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, xqpStringStore_t>, 6)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, var_expr_t>, 7)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<store::Item*, PrologOption>, 8)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, StaticallyKnownCollection_t, ItemPointerHashMapCmp>, 1)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, IndexDecl_t, ItemPointerHashMapCmp>, 2)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, ValueIC_t, ItemPointerHashMapCmp>, 3)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, function_t, ItemPointerHashMapCmp>, 4)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, std::vector<function_t>*, ItemPointerHashMapCmp>, 5)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, xqpStringStore_t, ItemPointerHashMapCmp>, 6)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, var_expr_t, ItemPointerHashMapCmp>, 7)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<store::Item*, PrologOption, ItemPointerHashMapCmp>, 8)

SERIALIZABLE_CLASS_VERSIONS(ItemPointerHashMapCmp)
END_SERIALIZABLE_CLASS_VERSIONS(ItemPointerHashMapCmp)

SERIALIZABLE_TEMPLATE_VERSIONS(serializable_ItemPointerHashMap)
END_SERIALIZABLE_TEMPLATE_VERSIONS(serializable_ItemPointerHashMap)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<StaticallyKnownCollection_t>, 1)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<IndexDecl_t>, 2)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<ValueIC_t>, 3)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<function_t>, 4)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<std::vector<function_t>* >, 5)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<xqpStringStore_t>, 6)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<var_expr_t>, 7)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_ItemPointerHashMap, serializable_ItemPointerHashMap<PrologOption>, 8)


// HashMapStrHandle
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<xqpStringStore_t, xqpStringStore_t>, 9)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<xqpStringStore_t, xqtref_t>, 10)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS2(serializable_HashEntry, serializable_HashEntry<xqpStringStore_t, static_context::ctx_module_t>, 11)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<xqpStringStore_t, xqpStringStore_t, HashMapStrHandleCmp>,9)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<xqpStringStore_t, xqtref_t, HashMapStrHandleCmp>, 10)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS3(serializable_HashMap, serializable_HashMap<xqpStringStore_t, static_context::ctx_module_t, HashMapStrHandleCmp>, 11)

SERIALIZABLE_CLASS_VERSIONS(HashMapStrHandleCmp)
END_SERIALIZABLE_CLASS_VERSIONS(HashMapStrHandleCmp)

SERIALIZABLE_TEMPLATE_VERSIONS(serializable_HashMapStrHandle)
END_SERIALIZABLE_TEMPLATE_VERSIONS(serializable_HashMapStrHandle)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_HashMapStrHandle, serializable_HashMapStrHandle<xqpStringStore_t>, 1)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_HashMapStrHandle, serializable_HashMapStrHandle<xqtref_t>, 2)

SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(serializable_HashMapStrHandle, serializable_HashMapStrHandle<static_context::ctx_module_t>, 3)
SERIALIZABLE_CLASS_VERSIONS(store::Iterator)
END_SERIALIZABLE_CLASS_VERSIONS(store::Iterator)




/*SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ElementIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FLWORIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GroupByIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<IfThenElseIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<AttributeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CompareIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnAdjustToTimeZoneIterator_2>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnBooleanIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LogicIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OpIsSameNodeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OpNodeBeforeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OpNodeAfterIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnRootIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnNodeNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnNilledIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnBaseUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDocumentUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaCollectionExistsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaNodeCountIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaNodeAtIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaIndexOfIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaExportXmlIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaListCollectionsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaImportXmlIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaImportCatalogIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaImportFolderIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaCreateCollectionIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaDeleteCollectionIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaDeleteAllCollectionsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaInsertNodeFirstIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaInsertNodeLastIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaInsertNodeBeforeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaInsertNodeAfterIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaInsertNodeAtIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRemoveNodeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRemoveNodeAtIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCurrentDateTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCurrentDateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCurrentTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnImplicitTimezoneIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDefaultCollationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaJSONSerializeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GenericArithIterator<AddOperation>>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GenericArithIterator<SubtractOperation>>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GenericArithIterator<MultiplyOperation>>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GenericArithIterator<DivideOperation>>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GenericArithIterator<IntegerDivideOperation>>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GenericArithIterator<ModOperation>>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<DocumentIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<DocumentContentIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CommentIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<PiIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<TextIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EnclosedIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<UDFunctionCallIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<StatelessExtFunctionCallIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<HoistIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<UnhoistIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EmptyIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SingletonIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NodeDistinctIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NodeSortIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<PathIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SelfAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<AttributeAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ParentAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<AncestorAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<AncestorSelfAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<RSiblingAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LSiblingAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ChildAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<DescendantAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<DescendantSelfAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<PrecedingAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FollowingAxisIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<InstanceOfIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CastIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CastableIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<PromoteIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<TreatIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EitherNodesOrAtomicsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<TryCatchIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<GetTupleFieldIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ForVarIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LetVarIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CountIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ForIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LetIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OuterForIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<TupleSourceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<TupleStreamIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<WhereIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDateTimeConstructorIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OpDurationEqualIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnYearsFromDurationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMonthsFromDurationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDaysFromDurationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnHoursFromDurationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMinutesFromDurationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSecondsFromDurationIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnYearFromDatetimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMonthFromDatetimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDayFromDatetimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnHoursFromDatetimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMinutesFromDatetimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSecondsFromDatetimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnTimezoneFromDatetimeIterato>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnYearFromDateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMonthFromDateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDayFromDateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnTimezoneFromDateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnHoursFromTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMinutesFromTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSecondsFromTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnTimezoneFromTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnFormatDateTimeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnAdjustToTimeZoneIterator_1>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnTraceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CtxVarAssignIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CtxVarExistsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EvalIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CreateValueIndex>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<DropValueIndex>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ValueIndexInsertSessionOpener>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ValueIndexInsertSessionCloser>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ValueIndexBuilder>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ValueIndexPointProbe>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnErrorIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnResolveUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SequentialIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FlowCtlIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LoopIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnReadStringIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnPrintIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NodeReferenceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NodeByReferenceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnLocalNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnNamespaceUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnLangIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCollectionIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OpNumericUnaryIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnAbsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCeilingIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnFloorIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnRoundIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnRoundHalfToEvenIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorNumGen>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSQRTIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnExpIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnLogIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSinIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCosIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnTanIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnArcSinIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnArcCosIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnArcTanIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ResolveQNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<QNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<QNameEqualIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<PrefixFromQNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LocalNameFromQNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NamespaceUriFromQNameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NamespaceUriForPrefixIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<InScopePrefixesIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRestGetIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRestPostIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRestPutIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRestDeleteIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRestHeadIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnConcatIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnIndexOfIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnEmptyIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnExistsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDistinctValuesIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnInsertBeforeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnRemoveIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnReverseIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSubsequenceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnZeroOrOneIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnOneOrMoreIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnExactlyOneIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDeepEqualIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<HashSemiJoinIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SortSemiJoinIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnCountIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnAvgIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMinMaxIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnSumIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<OpToIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnIdIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnIdRefIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDocIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnDocAvailableIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnParseIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CodepointsToStringIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<StringToCodepointsIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CompareStrIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<CodepointEqualIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ConcatStrIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<StringJoinIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SubstringIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<StringLengthIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NormalizeSpaceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<NormalizeUnicodeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<UpperCaseIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<LowerCaseIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<TranslateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EncodeForUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<IriToUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EscapeHtmlUriIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ContainsIterator)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<StartsWithIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<EndsWithIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SubstringBeforeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<SubstringAfterIterator)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnMatchesIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnReplaceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<FnTokenizeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<InsertIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<DeleteIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ReplaceIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<RenameIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaSchemaTypeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaBase64DecodeIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaBase64EncodeIterator>)
#ifdef ZORBA_WITH_TIDY
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaTidyIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaTDocIterator>)
#endif
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaRandomIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ZorbaUUIDIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<ValidateIterator>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(Batcher, Batcher<>)
*/

/*SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<AttributeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<CompareIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<FnAdjustToTimeZoneIterator_2, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<LogicIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<GenericArithIterator<AddOperation>, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<GenericArithIterator<SubtractOperation>, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<GenericArithIterator<MultiplyOperation>, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<GenericArithIterator<DivideOperation>, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<GenericArithIterator<IntegerDivideOperation>, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<GenericArithIterator<ModOperation>, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<PiIterator, PlanIteratorState >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<ForIterator, ForState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<LetIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<OuterForIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<TupleStreamIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<WhereIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<FnDateTimeConstructorIterator,PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<OpDurationEqualIterator,PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<ResolveQNameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<QNameIterator,PlanIteratorState >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<QNameEqualIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<InsertIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<ReplaceIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<RenameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(BinaryBaseIterator, BinaryBaseIterator<, >)
*/

/*SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<OpIsSameNodeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<OpNodeBeforeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<OpNodeAfterIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnRootIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnNodeNameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnNilledIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnBaseUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDocumentUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaCollectionExistsIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaNodeCountIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaNodeAtIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaIndexOfIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaExportXmlIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaListCollectionsIterator, ZorbaListCollectionsState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaImportXmlIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaImportCatalogIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaImportFolderIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaCreateCollectionIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaDeleteCollectionIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaDeleteAllCollectionsIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaInsertNodeFirstIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaInsertNodeLastIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaInsertNodeBeforeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaInsertNodeAfterIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaInsertNodeAtIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRemoveNodeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRemoveNodeAtIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCurrentDateTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCurrentDateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCurrentTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnImplicitTimezoneIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDefaultCollationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaJSONSerializeIterator, FnJSONSerializeIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<UDFunctionCallIterator, UDFunctionCallIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<StatelessExtFunctionCallIterator, StatelessExtFunctionCallIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<HoistIterator, HoistIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<UnhoistIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<TreatIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<EitherNodesOrAtomicsIterator, EitherNodesOrAtomicsIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<GetTupleFieldIterator, GetTupleFieldIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnYearsFromDurationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMonthsFromDurationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDaysFromDurationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnHoursFromDurationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMinutesFromDurationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSecondsFromDurationIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnYearFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMonthFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDayFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnHoursFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMinutesFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSecondsFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnTimezoneFromDatetimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnYearFromDateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMonthFromDateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDayFromDateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnTimezoneFromDateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnHoursFromTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMinutesFromTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSecondsFromTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnTimezoneFromTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnFormatDateTimeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator,
NaryBaseIterator<FnTraceIterator, FnTraceIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator,
NaryBaseIterator<CtxVarAssignIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<CtxVarExistsIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<EvalIterator, EvalIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ValueIndexBuilder, ValueIndexBuilderState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ValueIndexPointProbe, ValueIndexPointProbeState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnErrorIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnResolveUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<SequentialIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FlowCtlIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<LoopIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnReadStringIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnPrintIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<NodeReferenceIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<NodeByReferenceIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnLocalNameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnNamespaceUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnLangIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCollectionIterator, FnCollectionIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnAbsIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCeilingIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnFloorIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnRoundIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnRoundHalfToEvenIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSQRTIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnExpIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnLogIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSinIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCosIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnTanIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnArcSinIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnArcCosIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnArcTanIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<NamespaceUriForPrefixIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRestGetIterator, ZorbaRestGetIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRestPostIterator, ZorbaRestGetIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRestPutIterator, ZorbaRestGetIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRestDeleteIterator, ZorbaRestGetIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRestHeadIterator, ZorbaRestGetIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnConcatIterator, FnConcatIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnIndexOfIterator, FnIndexOfIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnEmptyIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnExistsIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDistinctValuesIterator, FnDistinctValuesIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnInsertBeforeIterator, FnInsertBeforeIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnRemoveIterator, FnRemoveIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnReverseIterator, FnReverseIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSubsequenceIterator, FnSubsequenceIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnZeroOrOneIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnOneOrMoreIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnExactlyOneIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDeepEqualIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<HashSemiJoinIterator, HashSemiJoinIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<SortSemiJoinIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnCountIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnAvgIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMinMaxIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnSumIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<OpToIterator, OpToIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnIdIterator, FnIdIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnIdRefIterator, FnIdRefIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnDocAvailableIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnParseIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<CodepointsToStringIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<StringToCodepointsIterator, StringToCodepointsState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<CompareStrIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<CodepointEqualIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ConcatStrIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<StringJoinIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<SubstringIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<StringLengthIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<NormalizeSpaceIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<NormalizeUnicodeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<UpperCaseIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<LowerCaseIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<TranslateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<EncodeForUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<IriToUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<EscapeHtmlUriIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ContainsIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<StartsWithIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<EndsWithIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<SubstringBeforeIterator PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<SubstringAfterIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnMatchesIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnReplaceIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<FnTokenizeIterator, FnTokenizeIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaSchemaTypeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaBase64DecodeIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaBase64EncodeIterator, PlanIteratorState>)
#ifdef ZORBA_WITH_TIDY
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaTidyIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaTDocIterator, PlanIteratorState>)
#endif
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaRandomIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<ZorbaUUIDIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NaryBaseIterator, NaryBaseIterator<, PlanIteratorState>)
*/

/*SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<EmptyIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<SingletonIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<ForVarIterator, ForVarState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<LetVarIterator, LetVarState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<TupleSourceIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<ZorNumGen, ZorNumGenState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(NoaryBaseIterator, NoaryBaseIterator<, >)
*/

/*SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<FnDataIterator, FnDataIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<FnBooleanIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<DocumentIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<DocumentContentIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<NameCastIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<CommentIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<TextIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<EnclosedIterator, EnclosedIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<NodeDistinctIterator, NodeDistinctState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<NodeSortIterator, NodeSortState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<PathIterator, PathIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<InstanceOfIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<CastIterator, CastIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<CastableIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<PromoteIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<TryCatchIterator, TryCatchIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<CountIterator, CountState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<FnAdjustToTimeZoneIterator_1,PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<CreateValueIndex, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<DropValueIndex, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<ValueIndexInsertSessionOpener, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<ValueIndexInsertSessionCloser, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<OpNumericUnaryIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<PrefixFromQNameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<LocalNameFromQNameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<NamespaceUriFromQNameIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<InScopePrefixesIterator,InScopePrefixesState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<FnDocIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<DeleteIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<ValidateIterator, PlanIteratorState>)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
SERIALIZABLE_TEMPLATE_INSTANCE_VERSIONS(UnaryBaseIterator, UnaryBaseIterator<, >)
*/


}
